@define-color base_color #2d2d2d;
@define-color text_color white;
@define-color bg_color #303030;
@define-color fg_color #f3f3f1;
@define-color selected_fg_color #ffffff;
@define-color selected_bg_color #0f3b71;
@define-color selected_borders_color black;
@define-color borders_color #686868;
@define-color alt_borders_color #4e4e4e;
@define-color borders_edge rgba(238, 238, 236, 0.07);
@define-color link_color #3584e4;
@define-color link_visited_color #1b6acb;
@define-color top_hilight rgba(238, 238, 236, 0.07);
@define-color dark_fill #282828;
@define-color headerbar_bg_color #2d2d2d;
@define-color menu_color #2d2d2d;
@define-color menu_selected_color #161616;
@define-color scrollbar_bg_color #313131;
@define-color scrollbar_slider_color #a4a4a3;
@define-color scrollbar_slider_hover_color #c9c9c7;
@define-color scrollbar_slider_active_color #1b6acb;
@define-color warning_color #f57900;
@define-color error_color #cc0000;
@define-color success_color #26ab62;
@define-color destructive_color #b2161d;
@define-color osd_fg_color #eeeeec;
@define-color osd_text_color white;
@define-color osd_bg_color rgba(38, 38, 38, 0.7);
@define-color osd_insensitive_bg_color rgba(58, 58, 57, 0.5);
@define-color osd_insensitive_fg_color #8a8a89;
@define-color osd_borders_color rgba(0, 0, 0, 0.7);
@define-color sidebar_bg_color #313131;
@define-color tooltip_borders_color rgba(255, 255, 255, 0.1);
@define-color shadow_color rgba(0, 0, 0, 0.1);
@define-color drop_target_color #26a269;
@define-color insensitive_fg_color #919191;
@define-color insensitive_bg_color #2f2f2f;
@define-color insensitive_borders_color #5d5d5d;
@define-color backdrop_base_color #303030;
@define-color backdrop_text_color #d6d6d6;
@define-color backdrop_bg_color #353535;
@define-color backdrop_fg_color #919190;
@define-color backdrop_insensitive_color #5b5b5b;
@define-color backdrop_selected_fg_color #d6d6d6;
@define-color backdrop_borders_color #202020;
@define-color backdrop_dark_fill #2e2e2e;
@define-color suggested_bg_color #15539e;
@define-color suggested_border_color #030c17;
@define-color progress_bg_color #15539e;
@define-color progress_border_color #030c17;
@define-color checkradio_bg_color #15539e;
@define-color checkradio_fg_color #ffffff;
@define-color checkradio_borders_color #092444;
@define-color switch_bg_color #15539e;
@define-color switch_borders_color #1f76e1;
@define-color focus_border_color rgba(255, 255, 255, 0.6);
@define-color alt_focus_border_color rgba(255, 255, 255, 0.6);
@define-color dim_label_opacity 0.9;
button { color: #f3f3f1; outline-color: rgba(255, 255, 255, 0.6); border-color: #686868; background-image: linear-gradient(to top, #323232 2px, #353535); box-shadow: 0 1px 2px rgba(0, 0, 0, 0.07); }

button:hover { color: #f3f3f1; border-color: #686868; background-image: linear-gradient(to top, #2b2b2b 20%, #2d2d2d 90%); box-shadow: 0 1px 2px rgba(0, 0, 0, 0.07); }

button:checked { color: #f3f3f1; border-color: #686868; background-image: image(#1e1e1e); box-shadow: none; }

button:checked:hover { color: #f3f3f1; border-color: #686868; background-image: image(#161616); box-shadow: none; }

button:checked:active { color: #f3f3f1; border-color: #686868; background-image: image(#111111); box-shadow: none; }

button:backdrop { color: #919190; border-color: #202020; background-image: image(#353535); box-shadow: none; }

button:backdrop:active { color: #919190; border-color: #202020; background-image: image(#2a2a2a); box-shadow: none; }

button:backdrop:insensitive { color: #5b5b5b; border-color: #202020; background-image: image(#2f2f2f); box-shadow: none; }

button:backdrop:insensitive:active { color: #5b5b5b; border-color: #202020; background-image: image(#2a2a2a); box-shadow: none; }

button:disabled { color: #919191; border-color: #5d5d5d; background-image: image(#2f2f2f); }

button:disabled:active { color: #919191; border-color: #5d5d5d; background-image: image(#252525); box-shadow: none; }

checkradio { background-clip: padding-box; background-image: linear-gradient(to bottom, #3c3c3c 20%, #303030 90%); border-color: #4e4e4e; box-shadow: 0 1px rgba(0, 0, 0, 0.05); color: #ffffff; }

checkradio:hover { background-image: linear-gradient(to bottom, #474747 10%, #3a3a3a 90%); }

checkradio:active { box-shadow: inset 0 1px black; background-image: image(#232323); }

checkradio:disabled { box-shadow: none; color: rgba(255, 255, 255, 0.7); }

checkradio:checked { background-clip: border-box; background-image: linear-gradient(to bottom, #185fb4 20%, #15539e 90%); border-color: #092444; box-shadow: 0 1px rgba(0, 0, 0, 0.05); color: #ffffff; }

checkradio:checked:hover { background-image: linear-gradient(to bottom, #1b68c6 10%, #185cb0 90%); }

checkradio:checked:active { box-shadow: inset 0 1px black; background-image: image(#124787); }

checkradio:checked:disabled { box-shadow: none; color: rgba(255, 255, 255, 0.7); }
